import EventEmitter from 'eventemitter3';
import { AddChild } from './AddChild';
import { Insert } from './Insert';
import { RemoveChild } from './RemoveChild';
/**
 * A list of commands that will be run to modify an [entity](crate::entity).
 */
export class EntityCommands extends EventEmitter {
    constructor(entity, commands) {
        super();
        this.entity = entity;
        this.commands = commands;
    }
    /**
     * Returns the [`Entity`] id of the entity.
     *
     * @example
     * let entity_id = commands.spawn_empty().id();
     */
    id() {
        return this.entity;
    }
    /**
     * Adds a [`Bundle`] of components to the entity.
     * This will overwrite any previous value(s) of the same component type.
     *
     * @example
     * commands
     *   .entity(0)
     *   .insert([Defense, { x: 10 }], [Transform, { translation } ])
     *   .insert([Bundle, { transform }])
     */
    insert(...bundles) {
        this.commands.add(new Insert(this.entity, bundles));
        return this;
    }
    /**
     * Removes a [`Bundle`] of components from the entity.
     * @example
     * commands.entity(0).remove(Transform);
     */
    remove(...components) {
        // this.commands.add(CommandsType.REMOVE, this.entity, null, ...components);
    }
    /**
     * Despawns the entity.
     */
    despawn() {
        // this.commands.add(CommandsType.DESPAWN, this.entity, null, null);
    }
    /**
     * Adds a single child.
     * If the children were previously children of another parent,
     * that parent's [`Children`] component will have those children removed from its list.
     * Removing all children from a parent causes its [`Children`] component to be removed from the entity.
     */
    appendChild(child) {
        this.commands.add(new AddChild(this.id(), child.id()));
        return this;
    }
    removeChild(child) {
        this.commands.add(new RemoveChild(this.id(), child.id()));
        return this;
    }
}
//# sourceMappingURL=EntityCommands.js.map