import { BufferUsage, BufferFrequencyHint, VertexStepMode, Format, } from '@antv/g-device-api';
import { uid } from '../utils';
import { Location } from '../shaders/wireframe';
import { FillGradient, FillImage, FillPattern, FillTexture, Wireframe, } from '../components';
// TODO: Use a more efficient way to manage Z index.
export const ZINDEX_FACTOR = 100000;
export class Drawcall {
    constructor(device, renderCache, texturePool, instanced, index) {
        this.device = device;
        this.renderCache = renderCache;
        this.texturePool = texturePool;
        this.instanced = instanced;
        this.index = index;
        this.uid = uid();
        this.shapes = [];
        /**
         * Create a new batch if the number of instances exceeds.
         */
        this.maxInstances = Infinity;
        this.geometryDirty = true;
        this.materialDirty = true;
        this.destroyed = false;
        this.vertexBuffers = [];
        this.vertexBufferDatas = [];
        this.vertexBufferOffsets = [];
        this.barycentricBufferDescriptor = {
            arrayStride: 4 * 3,
            stepMode: VertexStepMode.VERTEX,
            attributes: [
                {
                    shaderLocation: Location.BARYCENTRIC, // a_Barycentric
                    offset: 0,
                    format: Format.F32_RGB,
                },
            ],
        };
    }
    destroy() {
        var _a;
        if (this.program) {
            (_a = this.indexBuffer) === null || _a === void 0 ? void 0 : _a.destroy();
            this.vertexBuffers.forEach((buffer) => buffer.destroy());
            this.vertexBuffers = [];
            this.vertexBufferDatas = [];
            this.vertexBufferDescriptors = [];
        }
        this.destroyed = true;
    }
    validate(shape) {
        var _a, _b;
        if ((((_a = this.shapes[0]) === null || _a === void 0 ? void 0 : _a.has(Wireframe)) &&
            ((_b = this.shapes[0]) === null || _b === void 0 ? void 0 : _b.read(Wireframe).enabled)) !==
            (shape.has(Wireframe) && shape.read(Wireframe).enabled)) {
            return false;
        }
        return this.count() <= this.maxInstances - 1;
    }
    submit(renderPass, uniformBuffer, uniformLegacyObject) {
        if (this.geometryDirty) {
            this.createGeometry();
        }
        if (this.materialDirty) {
            let defines = '';
            if (this.instanced) {
                defines += '#define USE_INSTANCES\n';
            }
            if (this.useFillImage) {
                defines += '#define USE_FILLIMAGE\n';
            }
            if (this.useWireframe) {
                defines += '#define USE_WIREFRAME\n';
            }
            this.createMaterial(defines, uniformBuffer);
        }
        this.render(renderPass, uniformLegacyObject);
        if (this.geometryDirty) {
            this.geometryDirty = false;
        }
        if (this.materialDirty) {
            this.materialDirty = false;
        }
    }
    add(shape) {
        if (!this.shapes.includes(shape)) {
            this.shapes.push(shape);
            this.geometryDirty = true;
        }
    }
    remove(shape) {
        if (this.shapes.includes(shape)) {
            const index = this.shapes.indexOf(shape);
            this.shapes.splice(index, 1);
            this.geometryDirty = true;
        }
    }
    count() {
        return this.shapes.length;
    }
    get useWireframe() {
        var _a, _b;
        return (((_a = this.shapes[0]) === null || _a === void 0 ? void 0 : _a.has(Wireframe)) && ((_b = this.shapes[0]) === null || _b === void 0 ? void 0 : _b.read(Wireframe).enabled));
    }
    get useFillImage() {
        var _a;
        return (_a = this.shapes[0]) === null || _a === void 0 ? void 0 : _a.hasSomeOf(FillImage, FillTexture, FillGradient, FillPattern);
    }
    createProgram(vert, frag, defines) {
        const diagnosticDerivativeUniformityHeader = this.device.queryVendorInfo().platformString === 'WebGPU'
            ? 'diagnostic(off,derivative_uniformity);\n'
            : '';
        this.program = this.renderCache.createProgram({
            vertex: {
                glsl: defines + vert,
            },
            fragment: {
                glsl: defines + frag,
                postprocess: (fs) => diagnosticDerivativeUniformityHeader + fs,
            },
        });
        const vertexBufferDescriptors = this.vertexBufferDescriptors;
        if (this.useWireframe) {
            vertexBufferDescriptors.push(this.barycentricBufferDescriptor);
        }
        this.inputLayout = this.renderCache.createInputLayout({
            vertexBufferDescriptors,
            indexBufferFormat: Format.U32_R,
            program: this.program,
        });
    }
    generateWireframe() {
        const indiceNum = this.indexBufferData.length;
        const originalVertexBuffers = this.vertexBufferDatas.map((buffer) => {
            return buffer.slice();
        });
        for (let i = 0; i < this.vertexBufferDatas.length; i++) {
            const { arrayStride, stepMode } = this.vertexBufferDescriptors[i];
            if (stepMode === VertexStepMode.VERTEX) {
                this.vertexBufferDatas[i] = new Float32Array((arrayStride / Float32Array.BYTES_PER_ELEMENT) * indiceNum);
            }
            else {
                this.vertexBufferDatas[i] = originalVertexBuffers[i];
            }
        }
        // reallocate attribute data
        let cursor = 0;
        const uniqueIndices = new Uint32Array(indiceNum);
        for (let i = 0; i < indiceNum; i++) {
            const ii = this.indexBufferData[i];
            for (let j = 0; j < this.vertexBufferDatas.length; j++) {
                const { arrayStride, stepMode } = this.vertexBufferDescriptors[j];
                if (stepMode === VertexStepMode.VERTEX) {
                    const size = arrayStride / Float32Array.BYTES_PER_ELEMENT;
                    for (let k = 0; k < size; k++) {
                        this.vertexBufferDatas[j][cursor * size + k] =
                            originalVertexBuffers[j][ii * size + k];
                    }
                }
            }
            uniqueIndices[i] = cursor;
            cursor++;
        }
        for (let i = 0; i < this.vertexBuffers.length; i++) {
            this.vertexBuffers[i].destroy();
            this.vertexBuffers[i] = this.device.createBuffer({
                viewOrSize: this.vertexBufferDatas[i],
                usage: BufferUsage.VERTEX,
                hint: BufferFrequencyHint.DYNAMIC,
            });
        }
        // create barycentric attributes
        const barycentricBufferData = new Float32Array(indiceNum * 3);
        for (let i = 0; i < indiceNum;) {
            for (let j = 0; j < 3; j++) {
                const ii = uniqueIndices[i++];
                barycentricBufferData[ii * 3 + j] = 1;
            }
        }
        this.barycentricBuffer = this.device.createBuffer({
            viewOrSize: barycentricBufferData,
            usage: BufferUsage.VERTEX,
            hint: BufferFrequencyHint.DYNAMIC,
        });
        if (this.indexBuffer) {
            this.indexBuffer.destroy();
        }
        this.indexBuffer = this.device.createBuffer({
            viewOrSize: uniqueIndices,
            usage: BufferUsage.INDEX,
            hint: BufferFrequencyHint.STATIC,
        });
    }
}
//# sourceMappingURL=Drawcall.js.map