import { isNil } from '@antv/util';
import toposort from 'toposort';
import { Circle, Ellipse, FillSolid, Name, Opacity, Path, Polyline, Rect, Renderable, Stroke, Text, Transform, Visibility, } from '../..';
import { deserializePoints } from './points';
export function serializedNodesToEntities(nodes, commands) {
    var _a;
    const vertices = nodes.map((node) => node.id);
    const edges = nodes
        .filter((node) => !isNil(node.parentId))
        .map((node) => [node.parentId, node.id]);
    const sorted = toposort.array(vertices, edges);
    const idEntityMap = new Map();
    const entities = [];
    for (const id of sorted) {
        const node = nodes.find((node) => node.id === id);
        const { parentId, type } = node;
        const attributes = node;
        const entity = commands.spawn();
        idEntityMap.set(id, entity);
        const { transform } = attributes;
        entity.insert(new Transform(transform));
        if (type !== 'g') {
            entity.insert(new Renderable());
        }
        if (type === 'circle') {
            const { cx, cy, r } = attributes;
            entity.insert(new Circle({ cx, cy, r }));
        }
        else if (type === 'ellipse') {
            const { cx, cy, rx, ry } = attributes;
            entity.insert(new Ellipse({ cx, cy, rx, ry }));
        }
        else if (type === 'rect') {
            const { x, y, width, height, cornerRadius } = attributes;
            entity.insert(new Rect({ x, y, width, height, cornerRadius }));
        }
        else if (type === 'polyline') {
            const { points } = attributes;
            entity.insert(new Polyline({ points: deserializePoints(points) }));
        }
        else if (type === 'path') {
            const { d, fillRule, tessellationMethod } = attributes;
            entity.insert(new Path({ d, fillRule, tessellationMethod }));
        }
        else if (type === 'text') {
            const { x, y, content, fontFamily, fontSize, fontWeight, fontStyle, fontVariant, letterSpacing, lineHeight, whiteSpace, wordWrap, wordWrapWidth, textAlign, textBaseline, } = attributes;
            entity.insert(new Text({
                x,
                y,
                content,
                fontFamily,
                fontSize,
                fontWeight,
                fontStyle,
                fontVariant,
                letterSpacing,
                lineHeight,
                whiteSpace,
                wordWrap,
                wordWrapWidth,
                textAlign,
                textBaseline,
            }));
        }
        const { fill, fillOpacity, opacity } = attributes;
        if (fill) {
            entity.insert(new FillSolid(fill));
            // TODO: gradient, pattern, etc.
        }
        const { stroke, strokeWidth, strokeDasharray, strokeLinecap, strokeLinejoin, strokeMiterlimit, strokeOpacity, strokeDashoffset, strokeAlignment, } = attributes;
        if (stroke) {
            entity.insert(new Stroke({
                color: stroke,
                width: strokeWidth,
                dasharray: strokeDasharray === null || strokeDasharray === void 0 ? void 0 : strokeDasharray.split(',').map(Number),
                linecap: strokeLinecap,
                linejoin: strokeLinejoin,
                miterlimit: strokeMiterlimit,
                dashoffset: strokeDashoffset,
                alignment: strokeAlignment,
            }));
        }
        if (opacity || fillOpacity || strokeOpacity) {
            entity.insert(new Opacity({
                opacity,
                fillOpacity,
                strokeOpacity,
            }));
        }
        const { visibility } = attributes;
        entity.insert(new Visibility(visibility));
        const { name } = attributes;
        entity.insert(new Name(name));
        if (parentId) {
            (_a = idEntityMap.get(parentId)) === null || _a === void 0 ? void 0 : _a.appendChild(entity);
        }
        entities.push(entity.id().hold());
    }
    return { entities, idEntityMap };
}
//# sourceMappingURL=entity.js.map