/**
 * @see https://github.com/mapbox/mapbox-gl-js/blob/main/src/render/glyph_manager.ts
 */
import { Format, makeTextureDescriptor2D } from '@antv/g-device-api';
import { RGBAImage } from './alpha-image';
import { GlyphAtlas } from './glyph-atlas';
import { TinySDF } from './tiny-sdf';
import { DOMAdapter } from '../../environment';
/**
 * SDF_SCALE controls the pixel density of locally generated glyphs relative
 * to "normal" SDFs which are generated at 24pt font and a "pixel ratio" of 1.
 * The GlyphManager will generate glyphs SDF_SCALE times as large,
 * but with the same glyph metrics, and the quad generation code will scale them
 * back down so they display at the same size.
 *
 * The choice of SDF_SCALE is a trade-off between performance and quality.
 * Glyph generation time grows quadratically with the the scale, while quality
 * improvements drop off rapidly when the scale is higher than the pixel ratio
 * of the device. The scale of 2 buys noticeable improvements on HDPI screens
 * at acceptable cost.
 */
export const SDF_SCALE = 2;
export const BASE_FONT_WIDTH = 24 * SDF_SCALE;
export const BASE_FONT_BUFFER = 3 * SDF_SCALE;
export const RADIUS = 8 * SDF_SCALE;
export function getDefaultCharacterSet() {
    const charSet = [];
    for (let i = 32; i < 128; i++) {
        charSet.push(String.fromCharCode(i));
    }
    return charSet;
}
export class GlyphManager {
    constructor() {
        this.sdfGeneratorCache = {};
        this.glyphMap = {};
    }
    destroy() {
        if (this.glyphAtlasTexture) {
            this.glyphAtlasTexture.destroy();
        }
    }
    getMap() {
        return this.glyphMap;
    }
    getAtlas() {
        return this.glyphAtlas;
    }
    getAtlasTexture() {
        return this.glyphAtlasTexture;
    }
    layout(lines, fontStack, lineHeight, textAlign, letterSpacing, bitmapFont, scale, bitmapFontKerning, dx, dy) {
        const positionedGlyphs = [];
        let x = dx !== null && dx !== void 0 ? dx : 0;
        let y = dy !== null && dy !== void 0 ? dy : 0;
        const justify = textAlign === 'right' || textAlign === 'end'
            ? 1
            : textAlign === 'left' || textAlign === 'start'
                ? 0
                : 0.5;
        lines.forEach((line) => {
            const lineStartIndex = positionedGlyphs.length;
            let previousChar;
            DOMAdapter.get()
                .splitGraphemes(line)
                .forEach((char) => {
                let advance;
                let kerning = 0;
                if (bitmapFont) {
                    const charData = bitmapFont.chars[char];
                    advance = charData.xAdvance;
                    kerning =
                        (bitmapFontKerning &&
                            previousChar &&
                            charData.kerning[previousChar]) ||
                            0;
                }
                else {
                    const positions = this.glyphMap[fontStack];
                    const glyph = positions && positions[char];
                    advance = glyph.metrics.advance;
                }
                positionedGlyphs.push({
                    glyph: char,
                    x: x,
                    y: y,
                    scale,
                    fontStack,
                });
                x += (advance + kerning) * scale + letterSpacing;
                previousChar = char;
            });
            const lineWidth = x - letterSpacing;
            for (let i = lineStartIndex; i < positionedGlyphs.length; i++) {
                positionedGlyphs[i].x -= justify * lineWidth;
            }
            x = 0;
            y += lineHeight;
        });
        return positionedGlyphs;
    }
    generateAtlas(fontStack = '', fontFamily, fontWeight, fontStyle = '', text, device, esdt, fill) {
        let newChars = [];
        if (!this.glyphMap[fontStack]) {
            newChars = getDefaultCharacterSet();
        }
        const existedChars = Object.keys(this.glyphMap[fontStack] || {});
        Array.from(new Set(DOMAdapter.get().splitGraphemes(text))).forEach((char) => {
            if (existedChars.indexOf(char) === -1) {
                newChars.push(char);
            }
        });
        if (newChars.length) {
            const glyphMap = newChars
                .map((char) => {
                return this.generateSDF(fontStack, fontFamily, fontWeight, fontStyle, char, esdt, fill);
            })
                .reduce((prev, cur) => {
                prev[cur.id] = cur;
                return prev;
            }, {});
            // @ts-ignore
            this.glyphMap[fontStack] = Object.assign(Object.assign({}, this.glyphMap[fontStack]), glyphMap);
            this.glyphAtlas = new GlyphAtlas(this.glyphMap);
            const { width: atlasWidth, height: atlasHeight, data, } = this.glyphAtlas.image;
            if (this.glyphAtlasTexture) {
                this.glyphAtlasTexture.destroy();
            }
            this.glyphAtlasTexture = device.createTexture(Object.assign({}, makeTextureDescriptor2D(Format.U8_RGBA_NORM, atlasWidth, atlasHeight, 1)));
            this.glyphAtlasTexture.setImageData([data]);
        }
    }
    generateSDF(fontStack = '', fontFamily, fontWeight, fontStyle, char, esdt, fill) {
        let sdfGenerator = this.sdfGeneratorCache[fontStack + fill];
        if (!sdfGenerator) {
            sdfGenerator = this.sdfGeneratorCache[fontStack + fill] = new TinySDF({
                fontSize: BASE_FONT_WIDTH,
                fontFamily,
                fontWeight,
                fontStyle,
                buffer: BASE_FONT_BUFFER,
                radius: RADIUS,
                fill,
            });
        }
        // use sdf 2.x @see https://github.com/mapbox/tiny-sdf
        const { data, width, height, glyphWidth, glyphHeight, glyphLeft, glyphTop, glyphAdvance, } = sdfGenerator.draw(char, esdt, !!fill);
        return {
            id: char,
            // 在 canvas 中绘制字符，使用 Uint8Array 存储 30*30 sdf 数据
            bitmap: new RGBAImage({
                width,
                height,
            }, data),
            metrics: {
                width: glyphWidth / SDF_SCALE,
                height: glyphHeight / SDF_SCALE,
                left: glyphLeft / SDF_SCALE,
                top: glyphTop / SDF_SCALE,
                advance: glyphAdvance / SDF_SCALE,
            },
        };
    }
}
//# sourceMappingURL=glyph-manager.js.map