"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityCommands = void 0;
const eventemitter3_1 = __importDefault(require("eventemitter3"));
const AddChild_1 = require("./AddChild");
const Insert_1 = require("./Insert");
const RemoveChild_1 = require("./RemoveChild");
/**
 * A list of commands that will be run to modify an [entity](crate::entity).
 */
class EntityCommands extends eventemitter3_1.default {
    constructor(entity, commands) {
        super();
        this.entity = entity;
        this.commands = commands;
    }
    /**
     * Returns the [`Entity`] id of the entity.
     *
     * @example
     * let entity_id = commands.spawn_empty().id();
     */
    id() {
        return this.entity;
    }
    /**
     * Adds a [`Bundle`] of components to the entity.
     * This will overwrite any previous value(s) of the same component type.
     *
     * @example
     * commands
     *   .entity(0)
     *   .insert([Defense, { x: 10 }], [Transform, { translation } ])
     *   .insert([Bundle, { transform }])
     */
    insert(...bundles) {
        this.commands.add(new Insert_1.Insert(this.entity, bundles));
        return this;
    }
    /**
     * Removes a [`Bundle`] of components from the entity.
     * @example
     * commands.entity(0).remove(Transform);
     */
    remove(...components) {
        // this.commands.add(CommandsType.REMOVE, this.entity, null, ...components);
    }
    /**
     * Despawns the entity.
     */
    despawn() {
        // this.commands.add(CommandsType.DESPAWN, this.entity, null, null);
    }
    /**
     * Adds a single child.
     * If the children were previously children of another parent,
     * that parent's [`Children`] component will have those children removed from its list.
     * Removing all children from a parent causes its [`Children`] component to be removed from the entity.
     */
    appendChild(child) {
        this.commands.add(new AddChild_1.AddChild(this.id(), child.id()));
        return this;
    }
    removeChild(child) {
        this.commands.add(new RemoveChild_1.RemoveChild(this.id(), child.id()));
        return this;
    }
}
exports.EntityCommands = EntityCommands;
//# sourceMappingURL=EntityCommands.js.map