"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComputeCamera = void 0;
const becsy_1 = require("@lastolivegames/becsy");
const gl_matrix_1 = require("gl-matrix");
const components_1 = require("../components");
/**
 * Compute the points of the path according to the definition.
 */
class ComputeCamera extends becsy_1.System {
    constructor() {
        super();
        this.cameras = this.query((q) => q.current.addedOrChanged.with(components_1.Camera, components_1.Transform).trackWrites);
        this.canvases = this.query((q) => q.changed.with(components_1.Canvas).trackWrites);
        this.query((q) => q.using(components_1.ComputedCamera).write.and.using(components_1.Canvas).read);
    }
    execute() {
        this.cameras.addedOrChanged.forEach((entity) => {
            const camera = entity.read(components_1.Camera);
            if (!camera.canvas) {
                return;
            }
            const { width, height } = camera.canvas.read(components_1.Canvas);
            this.projection(entity, width, height);
            this.updateMatrix(entity);
            this.updateComputedCamera(entity);
        });
        this.canvases.changed.forEach((entity) => {
            const { cameras, width, height } = entity.read(components_1.Canvas);
            cameras.forEach((camera) => {
                this.projection(camera, width, height);
                this.updateComputedCamera(camera);
            });
        });
    }
    updateComputedCamera(entity) {
        const { translation, rotation, scale } = entity.read(components_1.Transform);
        Object.assign(entity.write(components_1.ComputedCamera), {
            x: translation.x,
            y: translation.y,
            rotation,
            zoom: 1 / scale.x,
        });
    }
    projection(camera, width, height) {
        if (!camera.has(components_1.ComputedCamera)) {
            camera.add(components_1.ComputedCamera);
        }
        const projectionMatrix = gl_matrix_1.mat3.projection(gl_matrix_1.mat3.create(), width, height);
        camera.write(components_1.ComputedCamera).projectionMatrix =
            components_1.Mat3.fromGLMat3(projectionMatrix);
        this.updateViewProjectionMatrix(camera);
    }
    updateMatrix(entity) {
        const { translation, rotation, scale } = entity.read(components_1.Transform);
        const viewMatrix = gl_matrix_1.mat3.create();
        gl_matrix_1.mat3.identity(viewMatrix);
        gl_matrix_1.mat3.translate(viewMatrix, viewMatrix, [translation.x, translation.y]);
        gl_matrix_1.mat3.rotate(viewMatrix, viewMatrix, rotation);
        gl_matrix_1.mat3.scale(viewMatrix, viewMatrix, [scale.x, scale.y]);
        gl_matrix_1.mat3.invert(viewMatrix, viewMatrix);
        entity.write(components_1.ComputedCamera).viewMatrix = components_1.Mat3.fromGLMat3(viewMatrix);
        this.updateViewProjectionMatrix(entity);
    }
    updateViewProjectionMatrix(camera) {
        const { projectionMatrix, viewMatrix } = camera.write(components_1.ComputedCamera);
        const viewProjectionMatrix = gl_matrix_1.mat3.multiply(gl_matrix_1.mat3.create(), components_1.Mat3.toGLMat3(projectionMatrix), components_1.Mat3.toGLMat3(viewMatrix));
        const viewProjectionMatrixInv = gl_matrix_1.mat3.invert(gl_matrix_1.mat3.create(), viewProjectionMatrix);
        Object.assign(camera.write(components_1.ComputedCamera), {
            viewProjectionMatrix: components_1.Mat3.fromGLMat3(viewProjectionMatrix),
            viewProjectionMatrixInv: components_1.Mat3.fromGLMat3(viewProjectionMatrixInv),
        });
    }
}
exports.ComputeCamera = ComputeCamera;
//# sourceMappingURL=ComputeCamera.js.map