"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComputePoints = void 0;
exports.shiftPoints = shiftPoints;
const becsy_1 = require("@lastolivegames/becsy");
const gl_matrix_1 = require("gl-matrix");
const components_1 = require("../components");
const utils_1 = require("../utils");
/**
 * Compute the points of the path according to the definition.
 */
class ComputePoints extends becsy_1.System {
    constructor() {
        super();
        this.paths = this.query((q) => q.addedOrChanged.with(components_1.Path).trackWrites);
        this.polylines = this.query((q) => q.addedOrChanged.with(components_1.Polyline).trackWrites);
        this.query((q) => q.current.with(components_1.ComputedPoints).write);
        this.query((q) => q.using(components_1.Stroke).read);
    }
    execute() {
        this.paths.addedOrChanged.forEach((entity) => {
            const { d } = entity.read(components_1.Path);
            const { subPaths } = (0, utils_1.parsePath)(d);
            const points = subPaths.map((subPath) => subPath
                .getPoints()
                .map((point) => [point[0], point[1]]));
            if (!entity.has(components_1.ComputedPoints)) {
                entity.add(components_1.ComputedPoints);
            }
            entity.write(components_1.ComputedPoints).points = points;
        });
        this.polylines.addedOrChanged.forEach((entity) => {
            const { points } = entity.read(components_1.Polyline);
            const { alignment, width } = entity.read(components_1.Stroke);
            const shiftedPoints = alignment === 'center'
                ? points
                : shiftPoints(points, alignment === 'inner', width);
            if (!entity.has(components_1.ComputedPoints)) {
                entity.add(components_1.ComputedPoints);
            }
            entity.write(components_1.ComputedPoints).shiftedPoints = shiftedPoints;
        });
    }
}
exports.ComputePoints = ComputePoints;
function shiftPoints(points, innerStrokeAlignment, strokeWidth) {
    const shiftedPoints = [];
    points.forEach(([x, y], index) => {
        const current = gl_matrix_1.vec2.fromValues(x, y);
        const prev = index === 0
            ? current
            : gl_matrix_1.vec2.fromValues(points[index - 1][0], points[index - 1][1]);
        const next = index === points.length - 1
            ? current
            : gl_matrix_1.vec2.fromValues(points[index + 1][0], points[index + 1][1]);
        const xBasis = gl_matrix_1.vec2.sub(gl_matrix_1.vec2.create(), current, prev);
        const len = gl_matrix_1.vec2.length(xBasis);
        const forward = gl_matrix_1.vec2.create();
        gl_matrix_1.vec2.divide(forward, xBasis, gl_matrix_1.vec2.fromValues(len, len));
        const norm = gl_matrix_1.vec2.fromValues(forward[1], -forward[0]);
        const xBasis2 = gl_matrix_1.vec2.sub(gl_matrix_1.vec2.create(), next, current);
        const len2 = gl_matrix_1.vec2.length(xBasis2);
        const forward2 = gl_matrix_1.vec2.create();
        gl_matrix_1.vec2.divide(forward2, xBasis2, gl_matrix_1.vec2.fromValues(len2, len2));
        const norm2 = gl_matrix_1.vec2.fromValues(forward2[1], -forward2[0]);
        const D = norm[0] * norm2[1] - norm[1] * norm2[0];
        // norm2 *= sign2;
        const lineAlignment = 2.0 * (innerStrokeAlignment ? 0 : 1) - 1.0;
        const shift = (strokeWidth / 2) * lineAlignment;
        if (len === 0) {
            gl_matrix_1.vec2.add(current, current, gl_matrix_1.vec2.scale(norm2, norm2, shift));
        }
        else if (len2 === 0) {
            gl_matrix_1.vec2.add(current, current, gl_matrix_1.vec2.scale(norm, norm, shift));
        }
        else {
            if (Math.abs(D) < 0.01) {
                gl_matrix_1.vec2.add(current, current, gl_matrix_1.vec2.scale(norm, norm, shift));
            }
            else {
                gl_matrix_1.vec2.add(current, current, (0, utils_1.bisect)(norm, norm2, shift));
            }
        }
        shiftedPoints.push([current[0], current[1]]);
    });
    return shiftedPoints;
}
//# sourceMappingURL=ComputePoints.js.map