"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropagateTransforms = exports.SyncSimpleTransforms = void 0;
const becsy_1 = require("@lastolivegames/becsy");
const components_1 = require("../components");
/**
 * Update {@link GlobalTransform} component of entities that aren't in the hierarchy
 * Third party plugins should ensure that this is used in concert with {@link PropagateTransforms}.
 */
class SyncSimpleTransforms extends becsy_1.System {
    constructor() {
        super();
        this.queries = this.query((q) => q.addedOrChanged.with(components_1.Transform).but.without(components_1.Parent, components_1.Children)
            .trackWrites);
        this.cameras = this.query((q) => q.addedOrChanged.with(components_1.Transform, components_1.Parent).without(components_1.Children).trackWrites);
        this.orphaned = this.query((q) => q.removed.with(components_1.Parent));
        this.query((q) => q.using(components_1.GlobalTransform).write);
    }
    syncTransform(entity, checkGlobalTransform = true) {
        const transform = entity.read(components_1.Transform);
        if (checkGlobalTransform && !entity.has(components_1.GlobalTransform)) {
            entity.add(components_1.GlobalTransform, new components_1.GlobalTransform());
        }
        const globalTransform = entity.write(components_1.GlobalTransform);
        globalTransform.from(transform);
    }
    execute() {
        this.cameras.addedOrChanged.forEach((entity) => {
            this.syncTransform(entity);
        });
        // Update changed entities.
        this.queries.addedOrChanged.forEach((entity) => {
            this.syncTransform(entity);
        });
        // Update orphaned entities.
        this.orphaned.removed.forEach((entity) => {
            this.syncTransform(entity, false);
        });
    }
}
exports.SyncSimpleTransforms = SyncSimpleTransforms;
/**
 * Update {@link GlobalTransform} component of entities based on entity hierarchy and {@link Transform} components.
 * Use `after` constraints here to ensure that the {@link SyncSimpleTransforms} system has run first.
 * @see https://lastolivegames.github.io/becsy/guide/architecture/systems#execution-order
 */
class PropagateTransforms extends becsy_1.System {
    constructor() {
        super();
        this.queries = this.query((q) => q
            .with(components_1.Transform, components_1.Parent)
            .addedOrChanged.trackWrites.using(components_1.GlobalTransform).write);
        this.query((q) => q.using(components_1.Camera).read);
    }
    execute() {
        this.queries.addedOrChanged.forEach((entity) => {
            // Camera's worldTransform reflects the camera's position in the world.
            // It is not affected by the hierarchy.
            const worldTransform = entity.has(components_1.Camera) || !entity.has(components_1.GlobalTransform)
                ? new components_1.GlobalTransform()
                : entity.read(components_1.GlobalTransform);
            entity.read(components_1.Parent).children.forEach((child) => {
                const localTransform = child.read(components_1.Transform);
                if (!child.has(components_1.GlobalTransform)) {
                    child.add(components_1.GlobalTransform, new components_1.GlobalTransform());
                }
                child.write(components_1.GlobalTransform).matrix = worldTransform.matrix.mul_mat3(components_1.Mat3.from_scale_angle_translation(localTransform.scale, localTransform.rotation, localTransform.translation));
            });
        });
    }
}
exports.PropagateTransforms = PropagateTransforms;
//# sourceMappingURL=Transform.js.map