"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Last = exports.PostUpdate = exports.Update = exports.PreUpdate = exports.First = exports.PostStartUp = exports.StartUp = exports.PreStartUp = void 0;
const becsy_1 = require("@lastolivegames/becsy");
/**
 * The schedule that contains the app logic that is evaluated each tick of [`App::update()`].
 *
 * By default, it will run the following schedules in the given order:
 *
 * On the first run of the schedule (and only on the first run), it will run:
 * * [`PreStartup`]
 * * [`Startup`]
 * * [`PostStartup`]
 *
 * Then it will run:
 * * [`First`]
 * * [`PreUpdate`]
 * * [`StateTransition`]
 * * [`RunFixedUpdateLoop`]
 *     * This will run [`FixedUpdate`] zero to many times, based on how much time has elapsed.
 * * [`Update`]
 * * [`PostUpdate`]
 * * [`Last`]
 */
/**
 * The schedule that runs before [`Startup`].
 */
exports.PreStartUp = becsy_1.System.group();
/**
 * The schedule that runs once when the app starts.
 */
exports.StartUp = becsy_1.System.group();
/**
 * The schedule that runs once after [`Startup`].
 */
exports.PostStartUp = becsy_1.System.group();
/**
 * Runs first in the schedule.
 */
exports.First = becsy_1.System.group();
/**
 * The schedule that contains logic that must run before [`Update`]. For example, a system that reads raw keyboard
 * input OS events into an `Events` resource. This enables systems in [`Update`] to consume the events from the `Events`
 * resource without actually knowing about (or taking a direct scheduler dependency on) the "os-level keyboard event system".
 *
 * [`PreUpdate`] exists to do "engine/plugin preparation work" that ensures the APIs consumed in [`Update`] are "ready".
 * [`PreUpdate`] abstracts out "pre work implementation details".
 *
 * See the [`Main`] schedule for some details about how schedules are run.
 */
exports.PreUpdate = becsy_1.System.group();
/**
 * The schedule that contains app logic.
 */
exports.Update = becsy_1.System.group();
/**
 * The schedule that contains logic that must run after [`Update`]. For example, synchronizing "local transforms" in a hierarchy
 * to "global" absolute transforms. This enables the [`PostUpdate`] transform-sync system to react to "local transform" changes in
 * [`Update`] without the [`Update`] systems needing to know about (or add scheduler dependencies for) the "global transform sync system".
 *
 * [`PostUpdate`] exists to do "engine/plugin response work" to things that happened in [`Update`].
 * [`PostUpdate`] abstracts out "implementation details" from users defining systems in [`Update`].
 *
 * See the [`Main`] schedule for some details about how schedules are run.
 */
exports.PostUpdate = becsy_1.System.group();
/**
 * Runs last in the schedule.
 */
exports.Last = becsy_1.System.group();
exports.PreStartUp.schedule((s) => s.before(exports.StartUp));
exports.StartUp.schedule((s) => s.before(exports.PostStartUp));
exports.PostStartUp.schedule((s) => s.before(exports.First));
exports.First.schedule((s) => s.before(exports.PreUpdate));
exports.PreUpdate.schedule((s) => s.before(exports.Update));
exports.Update.schedule((s) => s.before(exports.PostUpdate));
exports.PostUpdate.schedule((s) => s.before(exports.Last));
//# sourceMappingURL=stages.js.map