"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Path = void 0;
const gl_matrix_1 = require("gl-matrix");
const curve_path_1 = require("./curve-path");
const line_curve_1 = require("./line-curve");
const quadratic_bezier_curve_1 = require("./quadratic-bezier-curve");
const cubic_bezier_curve_1 = require("./cubic-bezier-curve");
const ellipse_curve_1 = require("./ellipse-curve");
class Path extends curve_path_1.CurvePath {
    constructor(points) {
        super();
        this.currentPoint = gl_matrix_1.vec2.create();
        if (points) {
            this.setFromPoints(points);
        }
    }
    setFromPoints(points) {
        this.moveTo(points[0][0], points[0][1]);
        for (let i = 1, l = points.length; i < l; i++) {
            this.lineTo(points[i][0], points[i][1]);
        }
        return this;
    }
    moveTo(x, y) {
        gl_matrix_1.vec2.set(this.currentPoint, x, y); // TODO consider referencing vectors instead of copying?
        return this;
    }
    lineTo(x, y) {
        const curve = new line_curve_1.LineCurve(gl_matrix_1.vec2.clone(this.currentPoint), gl_matrix_1.vec2.fromValues(x, y));
        this.curves.push(curve);
        gl_matrix_1.vec2.set(this.currentPoint, x, y);
        return this;
    }
    quadraticCurveTo(aCPx, aCPy, aX, aY) {
        const curve = new quadratic_bezier_curve_1.QuadraticBezierCurve(gl_matrix_1.vec2.clone(this.currentPoint), gl_matrix_1.vec2.fromValues(aCPx, aCPy), gl_matrix_1.vec2.fromValues(aX, aY));
        this.curves.push(curve);
        gl_matrix_1.vec2.set(this.currentPoint, aX, aY);
        return this;
    }
    bezierCurveTo(aCP1x, aCP1y, aCP2x, aCP2y, aX, aY) {
        const curve = new cubic_bezier_curve_1.CubicBezierCurve(gl_matrix_1.vec2.clone(this.currentPoint), gl_matrix_1.vec2.fromValues(aCP1x, aCP1y), gl_matrix_1.vec2.fromValues(aCP2x, aCP2y), gl_matrix_1.vec2.fromValues(aX, aY));
        this.curves.push(curve);
        gl_matrix_1.vec2.set(this.currentPoint, aX, aY);
        return this;
    }
    arc(aX, aY, aRadius, aStartAngle, aEndAngle, aClockwise) {
        const [x0, y0] = this.currentPoint;
        this.absarc(aX + x0, aY + y0, aRadius, aStartAngle, aEndAngle, aClockwise);
        return this;
    }
    absarc(aX, aY, aRadius, aStartAngle, aEndAngle, aClockwise) {
        this.absellipse(aX, aY, aRadius, aRadius, aStartAngle, aEndAngle, aClockwise);
        return this;
    }
    absellipse(aX, aY, xRadius, yRadius, aStartAngle, aEndAngle, aClockwise, aRotation) {
        const curve = new ellipse_curve_1.EllipseCurve(aX, aY, xRadius, yRadius, aStartAngle, aEndAngle, aClockwise, aRotation);
        if (this.curves.length > 0) {
            // if a previous curve is present, attempt to join
            const firstPoint = curve.getPoint(0);
            if (!gl_matrix_1.vec2.equals(firstPoint, this.currentPoint)) {
                this.lineTo(firstPoint[0], firstPoint[1]);
            }
        }
        this.curves.push(curve);
        const lastPoint = curve.getPoint(1);
        gl_matrix_1.vec2.copy(this.currentPoint, lastPoint);
        return this;
    }
}
exports.Path = Path;
//# sourceMappingURL=path.js.map