"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.svgElementsToSerializedNodes = svgElementsToSerializedNodes;
exports.kebabToCamelCase = kebabToCamelCase;
const serialize_1 = require("../serialize");
const points_1 = require("./points");
/**
 * Note that this conversion is not fully reversible.
 * For example, in the StrokeAlignment implementation, one Circle corresponds to two <circle>s.
 * The same is true in Figma.
 *
 * @see https://github.com/ShukantPal/pixi-essentials/blob/master/packages/svg
 */
function svgElementsToSerializedNodes(elements, id = 0, defsChildren = [], parentId) {
    const nodes = [];
    for (const element of elements) {
        let type = element.tagName.toLowerCase();
        if (type === 'svg') {
            type = 'g';
        }
        else if (type === 'defs') {
            defsChildren.push(...Array.from(element.childNodes));
            continue;
        }
        else if (type === 'use') {
            const href = element.getAttribute('xlink:href');
            if (href) {
                const def = defsChildren.find((d) => d.id === href.replace('#', ''));
                if (def) {
                    return svgElementsToSerializedNodes([def], id, defsChildren, parentId);
                }
            }
            continue;
        }
        else if (type === 'tspan') {
            continue;
        }
        const attributes = Array.from(element.attributes).reduce((prev, attr) => {
            let attributeName = kebabToCamelCase(attr.name);
            let value = attr.value;
            if (attributeName === 'transform') {
                value = (0, serialize_1.serializeTransform)(value);
            }
            else if (type === 'rect' &&
                (attributeName === 'rx' || attributeName === 'ry')) {
                attributeName = 'radius';
                value = Number(value);
            }
            else if (attributeName === 'cx' ||
                attributeName === 'cy' ||
                attributeName === 'x' ||
                attributeName === 'y' ||
                attributeName === 'rx' ||
                attributeName === 'ry' ||
                attributeName === 'r' ||
                attributeName === 'width' ||
                attributeName === 'height' ||
                attributeName === 'opacity' ||
                attributeName === 'fillOpacity' ||
                attributeName === 'strokeOpacity' ||
                attributeName === 'strokeWidth' ||
                attributeName === 'strokeMiterlimit' ||
                attributeName === 'strokeDashoffset' ||
                attributeName === 'fontSize') {
                // remove 'px' suffix
                value = Number(value.replace('px', ''));
            }
            else if (attributeName === 'textAnchor') {
                attributeName = 'textAlign';
                if (value === 'middle') {
                    value = 'center';
                }
            }
            prev[attributeName] = value;
            return prev;
        }, {});
        if (type === 'text') {
            attributes.content = element.textContent;
        }
        else if (type === 'line') {
            type = 'polyline';
            // @ts-ignore
            const { x1, y1, x2, y2 } = attributes;
            attributes.points = `${x1},${y1} ${x2},${y2}`;
            // @ts-ignore
            delete attributes.x1;
            // @ts-ignore
            delete attributes.y1;
            // @ts-ignore
            delete attributes.x2;
            // @ts-ignore
            delete attributes.y2;
        }
        else if (type === 'polygon') {
            type = 'path';
            // @ts-ignore
            const points = (0, points_1.deserializePoints)(attributes.points);
            let d = '';
            points.forEach((point, index) => {
                if (index === 0) {
                    d += `M${point[0]},${point[1]}`;
                }
                else {
                    d += `L${point[0]},${point[1]}`;
                }
            });
            d += 'Z';
            attributes.d = d;
            // @ts-ignore
            delete attributes.points;
        }
        // @see https://developer.mozilla.org/en-US/docs/Web/SVG/Reference/Attribute/visibility
        const visibility = element.getAttribute('visibility') || 'visible';
        attributes.visibility = visibility;
        attributes.name =
            element.getAttribute('name') ||
                (type === 'text' && attributes.content) ||
                `Layer ${id}`;
        const node = Object.assign({ id: `${id}`, parentId: parentId ? `${parentId}` : undefined, type }, attributes);
        nodes.push(node);
        const children = svgElementsToSerializedNodes(Array.from(element.children), ++id, defsChildren, Number(node.id)).filter(Boolean);
        id += children.length;
        nodes.push(...children);
    }
    return nodes;
}
function kebabToCamelCase(str) {
    return str.replace(/-([a-z])/g, (match, letter) => letter.toUpperCase());
}
//# sourceMappingURL=svg.js.map